@@ -4,16 +4,18 @@ from slither.core.source_mapping.source_mapping import SourceMapping
 
 if TYPE_CHECKING:
     from slither.core.variables.structure_variable import StructureVariable
+    from slither.core.compilation_unit import SlitherCompilationUnit
 
 
 class Structure(SourceMapping):
-    def __init__(self):
+    def __init__(self, compilation_unit: "SlitherCompilationUnit"):
         super().__init__()
         self._name = None
         self._canonical_name = None
         self._elems: Dict[str, "StructureVariable"] = dict()
         # Name of the elements in the order of declaration
         self._elems_ordered: List[str] = []
+        self.compilation_unit = compilation_unit
 
     @property
     def canonical_name(self) -> str:
