@@ -20,7 +20,7 @@ class Backdoor(AbstractDetector):
     def _detect(self):
         results = []
 
-        for contract in self.slither.contracts_derived:
+        for contract in self.compilation_unit.contracts_derived:
             # Check if a function has 'backdoor' in its name
             for f in contract.functions:
                 if "backdoor" in f.name:
