@@ -50,6 +50,9 @@ class Assembly(AbstractDetector):
             for func, nodes in values:
                 info = [func, " uses assembly\n"]
 
+                # sort the nodes to get deterministic results
+                nodes.sort(key=lambda x: x.node_id)
+
                 for node in nodes:
                     info += ["\t- ", node, "\n"]
 
