@@ -9,12 +9,12 @@ from slither.core.declarations import (
     SolidityVariable,
     Structure,
 )
+from slither.core.declarations.solidity_import_placeholder import SolidityImportPlaceHolder
 from slither.core.solidity_types.type import Type
 from slither.core.variables.local_variable import LocalVariable
 from slither.core.variables.state_variable import StateVariable
 from slither.slithir.operations import (
     Assignment,
-    Balance,
     Binary,
     Condition,
     Delete,
@@ -173,7 +173,9 @@ def add_ssa_ir(function, all_state_variables_instances):
     init_state_variables_instances = dict(all_state_variables_instances)
 
     initiate_all_local_variables_instances(
-        function.nodes, init_local_variables_instances, all_init_local_variables_instances,
+        function.nodes,
+        init_local_variables_instances,
+        all_init_local_variables_instances,
     )
 
     generate_ssa_irs(
@@ -281,7 +283,13 @@ def generate_ssa_irs(
                         if isinstance(new_ir.rvalue, ReferenceVariable):
                             refers_to = new_ir.rvalue.points_to_origin
                             new_ir.lvalue.add_refers_to(refers_to)
-                        else:
+                        # Discard Constant
+                        # This can happen on yul, like
+                        # assembly { var.slot = some_value }
+                        # Here we do not keep track of the references as we do not track
+                        # such low level manipulation
+                        # However we could extend our storage model to do so in the future
+                        elif not isinstance(new_ir.rvalue, Constant):
                             new_ir.lvalue.add_refers_to(new_ir.rvalue)
 
     for succ in node.dominator_successors:
@@ -506,7 +514,8 @@ def add_phi_origins(node, local_variables_definition, state_variables_definition
     # We keep the instance as we want to avoid to add __hash__ on v.name in Variable
     # That might work for this used, but could create collision for other uses
     local_variables_definition = dict(
-        local_variables_definition, **{v.name: (v, node) for v in node.local_variables_written},
+        local_variables_definition,
+        **{v.name: (v, node) for v in node.local_variables_written},
     )
     state_variables_definition = dict(
         state_variables_definition,
@@ -598,7 +607,17 @@ def get(
         return tuple_variables_instances[variable.index]
     assert isinstance(
         variable,
-        (Constant, SolidityVariable, Contract, Enum, SolidityFunction, Structure, Function, Type,),
+        (
+            Constant,
+            SolidityVariable,
+            Contract,
+            Enum,
+            SolidityFunction,
+            Structure,
+            Function,
+            Type,
+            SolidityImportPlaceHolder,
+        ),
     )  # type for abi.decode(.., t)
     return variable
 
@@ -650,10 +669,6 @@ def copy_ir(ir, *instances):
         rvalue = get_variable(ir, lambda x: x.rvalue, *instances)
         variable_return_type = ir.variable_return_type
         return Assignment(lvalue, rvalue, variable_return_type)
-    if isinstance(ir, Balance):
-        lvalue = get_variable(ir, lambda x: x.lvalue, *instances)
-        value = get_variable(ir, lambda x: x.value, *instances)
-        return Balance(value, lvalue)
     if isinstance(ir, Binary):
         lvalue = get_variable(ir, lambda x: x.lvalue, *instances)
         variable_left = get_variable(ir, lambda x: x.variable_left, *instances)
