@@ -1,4 +1,5 @@
 from slither.core.expressions import BinaryOperationType, Literal
+from slither.utils.integer_conversion import convert_string_to_int
 from slither.visitors.expression.expression import ExpressionVisitor
 
 
@@ -36,7 +37,7 @@ class ConstantFolding(ExpressionVisitor):
         if not isinstance(expr, Literal):
             cf = ConstantFolding(expr, self._type)
             expr = cf.result()
-        set_val(expression, int(expr.value))
+        set_val(expression, convert_string_to_int(expr.value))
 
     def _post_binary_operation(self, expression):
         left = get_val(expression.expression_left)
